@@ -32,9 +32,11 @@ import java.net.URL;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.AbstractMap;
+import java.util.AbstractSet;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Enumeration;
+import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
@@ -169,7 +171,7 @@ public class Properties extends AbstractMap<String, String> {
      *
      * @param os an output stream.
      * @param comment this parameter is ignored as this Properties
-     * @throws IOException
+     * @throws IOException If storing fails
      */
     public void store(OutputStream os, String comment) throws IOException {
         this.save(os);
@@ -192,7 +194,7 @@ public class Properties extends AbstractMap<String, String> {
      *
      * @param key the property key.
      * @param defaultValue a default value.
-     * @return
+     * @return The property value of the default value
      */
     public String getProperty(String key, String defaultValue) {
         if (this.get(key) != null)
@@ -202,7 +204,46 @@ public class Properties extends AbstractMap<String, String> {
 
     @Override
     public Set<Entry<String, String>> entrySet() {
-        return storage.entrySet();
+        return new AbstractSet<Entry<String, String>>() {
+            @Override
+            public Iterator<Entry<String, String>> iterator() {
+                return new Iterator<Entry<String, String>>() {
+                    final Iterator<Entry<String, String>> keyIterator = storage.entrySet().iterator();
+                    public boolean hasNext() {
+                        return keyIterator.hasNext();
+                    }
+                    public Entry<String, String> next() {
+                        final Entry<String, String> entry = keyIterator.next();
+                        return new Entry<String, String>() {
+                            public String getKey() {
+                                return entry.getKey();
+                            }
+                            public String getValue() {
+                                return entry.getValue();
+                            }
+                            public String setValue(String value) {
+                                String old = entry.setValue(value);
+                                if (old == null || !old.equals(value)) {
+                                    Layout l = layout.get(entry.getKey());
+                                    if (l != null) {
+                                        l.clearValue();
+                                    }
+                                }
+                                return old;
+                            }
+                        };
+                    }
+                    public void remove() {
+                        keyIterator.remove();
+                    }
+                };
+            }
+
+            @Override
+            public int size() {
+                return storage.size();
+            }
+        };
     }
 
     /**
@@ -243,21 +284,32 @@ public class Properties extends AbstractMap<String, String> {
         commentLines = new ArrayList<String>(commentLines);
         valueLines = new ArrayList<String>(valueLines);
         String escapedKey = escapeKey(key);
+        StringBuilder sb = new StringBuilder();
         int lastLine = valueLines.size() - 1;
         if (valueLines.isEmpty()) {
             valueLines.add(escapedKey + "=");
-        } else if (!valueLines.get(0).trim().startsWith(escapedKey)) {
-            valueLines.set(0, escapedKey + " = " + escapeJava(valueLines.get(0)) + (0 < lastLine? "\\": ""));
+            sb.append(escapedKey).append("=");
+        } else {
+            String val0 = valueLines.get(0);
+            if (!val0.trim().startsWith(escapedKey)) {
+                valueLines.set(0, escapedKey + " = " + escapeJava(val0) /*+ (0 < lastLine? "\\": "")*/);
+                sb.append(escapedKey).append(" = ").append(escapeJava(val0));
+            } else {
+                valueLines.set(0, escapeJava(val0) /*+ (0 < lastLine? "\\": "")*/);
+                sb.append(escapeJava(val0));
+            }
         }
         for (int i = 1; i < valueLines.size(); i++) {
-            valueLines.set(i, escapeJava(valueLines.get(i)) + (i < lastLine? "\\": ""));
-        }
-        StringBuilder value = new StringBuilder();
-        for (String line: valueLines) {
-            value.append(line);
+            String val = valueLines.get(i);
+            valueLines.set(i, escapeJava(val) /*+ (i < lastLine? "\\": "")*/);
+            while (val.length() > 0 && Character.isWhitespace(val.charAt(0))) {
+                val = val.substring(1);
+            }
+            sb.append(val);
         }
+        String[] property = PropertiesReader.parseProperty(sb.toString());
         this.layout.put(key, new Layout(commentLines, valueLines));
-        return storage.put(key, unescapeJava(value.toString()));
+        return storage.put(key, property[1]);
     }
 
     public String put(String key, List<String> commentLines, String value) {
@@ -270,6 +322,47 @@ public class Properties extends AbstractMap<String, String> {
         return put(key, Collections.singletonList(comment), value);
     }
 
+    public boolean update(Map<String, String> props) {
+        Properties properties;
+        if (props instanceof Properties) {
+            properties = (Properties) props;
+        } else {
+            properties = new Properties();
+            for (Map.Entry<? extends String, ? extends String> e : props.entrySet()) {
+                properties.put(e.getKey(), e.getValue());
+            }
+        }
+        return update(properties);
+    }
+
+    public boolean update(Properties properties) {
+        boolean modified = false;
+        // Remove "removed" properties from the cfg file
+        for (String key : new ArrayList<String>(this.keySet())) {
+            if (!properties.containsKey(key)) {
+                this.remove(key);
+                modified = true;
+            }
+        }
+        // Update existing keys
+        for (String key : properties.keySet()) {
+            String v = this.get(key);
+            List<String> comments = properties.getComments(key);
+            List<String> value = properties.getRaw(key);
+            if (v == null) {
+                this.put(key, comments, value);
+                modified = true;
+            } else if (!v.equals(properties.get(key))) {
+                if (comments.isEmpty()) {
+                    comments = this.getComments(key);
+                }
+                this.put(key, comments, value);
+                modified = true;
+            }
+        }
+        return modified;
+    }
+
     public List<String> getRaw(String key) {
         if (layout.containsKey(key)) {
             if (layout.get(key).getValueLines() != null) {
@@ -283,6 +376,15 @@ public class Properties extends AbstractMap<String, String> {
         return result;
     }
 
+    public List<String> getComments(String key) {
+        if (layout.containsKey(key)) {
+            if (layout.get(key).getCommentLines() != null) {
+                return new ArrayList<String>(layout.get(key).getCommentLines());
+            }
+        }
+        return new ArrayList<String>();
+    }
+
     @Override
     public String remove(Object key) {
         Layout l = layout.get(key);
@@ -350,8 +452,10 @@ public class Properties extends AbstractMap<String, String> {
     protected void loadLayout(Reader in) throws IOException
     {
         PropertiesReader reader = new PropertiesReader(in);
+        boolean hasProperty = false;
         while (reader.nextProperty())
         {
+            hasProperty = true;
             storage.put(reader.getPropertyName(), reader.getPropertyValue());
             int idx = checkHeaderComment(reader.getCommentLines());
             layout.put(reader.getPropertyName(),
@@ -360,7 +464,11 @@ public class Properties extends AbstractMap<String, String> {
                                     null,
                                new ArrayList<String>(reader.getValueLines())));
         }
-        footer = new ArrayList<String>(reader.getCommentLines());
+        if (hasProperty) {
+            footer = new ArrayList<String>(reader.getCommentLines());
+        } else {
+            header = new ArrayList<String>(reader.getCommentLines());
+        }
         if (substitute)
         {
             substitute();
@@ -414,9 +522,17 @@ public class Properties extends AbstractMap<String, String> {
             }
             if (l != null && l.getValueLines() != null)
             {
-                for (String s : l.getValueLines())
+                for (int i = 0; i < l.getValueLines().size(); i++)
                 {
-                    writer.writeln(s);
+                    String s = l.getValueLines().get(i);
+                    if (i < l.getValueLines().size() - 1)
+                    {
+                        writer.writeln(s + "\\");
+                    }
+                    else
+                    {
+                        writer.writeln(s);
+                    }
                 }
             }
             else
@@ -582,12 +698,12 @@ public class Properties extends AbstractMap<String, String> {
      * <p>The only difference between Java strings and JavaScript strings
      * is that in JavaScript, a single quote must be escaped.</p>
      *
-     * <p>Example:
+     * <p>Example:</p>
      * <pre>
      * input string: He didn't say, "Stop!"
      * output string: He didn't say, \"Stop!\"
      * </pre>
-     * </p>
+     *
      *
      * @param str  String to escape values in, may be null
      * @return String with escaped values, <code>null</code> if null string input
@@ -782,20 +898,19 @@ public class Properties extends AbstractMap<String, String> {
                     continue;
                 }
 
+                boolean combine = checkCombineLines(line);
+                if (combine)
+                {
+                    line = line.substring(0, line.length() - 1);
+                }
                 valueLines.add(line);
                 while (line.length() > 0 && contains(WHITE_SPACE, line.charAt(0)))
                 {
                     line = line.substring(1, line.length());
                 }
-
-                if (checkCombineLines(line))
+                buffer.append(line);
+                if (!combine)
                 {
-                    line = line.substring(0, line.length() - 1);
-                    buffer.append(line);
-                }
-                else
-                {
-                    buffer.append(line);
                     break;
                 }
             }
@@ -1099,4 +1214,4 @@ public class Properties extends AbstractMap<String, String> {
 
     } // class Layout
 
-}
\ No newline at end of file
+}
